//
//  AppDelegate.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 22/10/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit
//typealias SubstancesCollection  = [Reactant]
//typealias ExperimentCollection = [Experiment]

@UIApplicationMain
class AppDelegate: UIResponder, UIApplicationDelegate {
    
    var window: UIWindow?
    var currentSession = Session()
    var substances = [Solution]()
    var pastSessions = [Session]()
    var experimentTypes = [Experiment]()
    
    enum fileNames  {
        static let currentSession = "CurrentSession.json"
        static let pastSessions = "PastSessions.json"
        static let substances = "Substances.json"
        static let experimentTypes = "ExperimentTypes.json"
        
    }
    
    
    func application(_ application: UIApplication, didFinishLaunchingWithOptions launchOptions: [UIApplicationLaunchOptionsKey: Any]?) -> Bool {
        // Override point for customization after application launch.
        loadState()
        if substances.count == 0 && pastSessions.count == 0{
            loadBundleFiles()
        }
        return true
    }
    
    func applicationWillResignActive(_ application: UIApplication) {
        // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
        // Use this method to pause ongoing tasks, disable timers, and invalidate graphics rendering callbacks. Games should use this method to pause the game.
        
    }
    
    func applicationDidEnterBackground(_ application: UIApplication) {
        // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
        // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
        
        saveState()
        
        
        
    }
    
    func applicationWillEnterForeground(_ application: UIApplication) {
        // Called as part of the transition from the background to the active state; here you can undo many of the changes made on entering the background.
        
        loadState()
    }
    
    func applicationDidBecomeActive(_ application: UIApplication) {
        // Restart any tasks that were paused (or not yet started) while the application was inactive. If the application was previously in the background, optionally refresh the user interface.
    }
    
    func applicationWillTerminate(_ application: UIApplication) {
        // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
        saveState()
    }
    
    
   
    func saveState() {
        // Let's save up the experiment and status in the Documents directory
        print("Saving Data")
        // We define the filenames to save
        let currentSession = fileNames.currentSession
        let pastSessions = fileNames.pastSessions
        let substances = fileNames.substances
        let experimentTypes = fileNames.experimentTypes
        // Build a dictionary to assing KeyPaths to each fileName
        let filesToSave = [
            currentSession : \AppDelegate.currentSession
            , pastSessions : \AppDelegate.pastSessions
            , substances : \AppDelegate.substances
            , experimentTypes : \AppDelegate.experimentTypes
        ]
        
        let defaults = UserDefaults.standard
        let encoder = JSONEncoder()
        encoder.dateEncodingStrategy = .iso8601
        
        // Iterate through the dictionary and save the JSON
        for (name, keyPath) in filesToSave{
            // save the data in
            let toSave = self[keyPath:keyPath]
            let fileName = name
            var data = Data()
            
            do {
                switch fileName{
                case currentSession:
                    data = try encoder.encode(toSave as! Session)
                case pastSessions:
                    data = try encoder.encode(toSave as! [Session])
                case substances:
                    data = try encoder.encode(toSave as! [Solution])
                case experimentTypes:
                    data = try encoder.encode(toSave as! [Experiment])
                default:
                    break
}
            }catch {
                print("Error encoding :\(fileName)")
            }
            
            
            if let url = self.saveJsonFile(fileName, withData: data) {
                defaults.set(url.absoluteString, forKey: fileName)
                print("Saved : \(fileName) at \(url.path)")
            } else {
                print("Failed to save \(fileName)")
            }
        }
    }
    
    func loadState(){
        print("Loading DataStructures")
        let currentSession = fileNames.currentSession
        let pastSessions = fileNames.pastSessions
        let substances = fileNames.substances
        let experimentTypes = fileNames.experimentTypes
        // Build a dictionary to assing KeyPaths to each fileName
        let dataFiles = [
            currentSession
            , pastSessions
            , substances
            , experimentTypes
        ]
//        let defaults = UserDefaults.standard
        
        for name in dataFiles{
            // save the data in
            let fileName = name
            
            let fileM = FileManager.default
            
            let directoriesURL = fileM.urls(for: .documentDirectory, in: .userDomainMask)

//            print("\ndirectoriesURL: \(directoriesURL.description)")
            
            let fileURL = directoriesURL.first?.appendingPathComponent(fileName)
//            print("\nfileURL: \(fileURL?.description)")
//            print("URL valid for :\(fileName)")
            do {
                let file = try FileHandle(forReadingFrom: fileURL!)
                let data = file.readDataToEndOfFile()
                let decoder = JSONDecoder()
                decoder.dateDecodingStrategy = .iso8601
                switch fileName{
                case currentSession:
                    self[keyPath:\AppDelegate.currentSession] = try decoder.decode(Session.self, from: data)
//                    print("Loading currentSession:")
                case pastSessions:
                    self[keyPath:\AppDelegate.pastSessions] = try decoder.decode([Session].self, from: data)
                case substances:
                    self[keyPath:\AppDelegate.substances] = try decoder.decode([Solution].self, from: data)
                case experimentTypes:
                    self[keyPath:\AppDelegate.experimentTypes] = try decoder.decode([Experiment].self, from: data)
                default:
                    break
                }
                
                //                    print("Successfully loaded: \(name)")
                
            } catch  {
                print("Error while trying to access the saved JSON File for \(name)")
            }
            
        }
    }
    
    func saveJsonFile(_ name:String, withData data:Data) -> URL? {
        let documentsDirectoryPathString = NSSearchPathForDirectoriesInDomains(.documentDirectory, .userDomainMask, true).first!
        let documentsDirectoryPath = NSURL(string: documentsDirectoryPathString)!
        let jsonFilePath = documentsDirectoryPath.appendingPathComponent(name)!
        
        let fileManager = FileManager.default
//        var isDirectory: ObjCBool = false
        var result : URL?
        // creating a .json file in the Documents folder
//        if !fileManager.fileExists(atPath: jsonFilePath.absoluteString, isDirectory: &isDirectory) {
            let created = fileManager.createFile(atPath: jsonFilePath.absoluteString, contents: nil, attributes: nil)
            if created {
                //                print("File created, let's fill it up")
                let jsonString =  String(data: data, encoding: .utf8)
                print("Json created with contents:\n\(String(describing: jsonString))\n")
                
                // Write that JSON to the file created earlier
                do {
                    let file = try FileHandle(forWritingTo: jsonFilePath)
                    file.write(data)
                    print("JSON data was written to the file successfully!")
                    result = jsonFilePath
                } catch let error as NSError {
                    print("Couldn't write to file:\n \(error.localizedDescription)\n")
                }
            } else {
                print("Couldn't create file for some reason")
            }
//        } else {
//            //            print("File already exists")
//            // then we have to update the contents of the file instead
//            do {
//                let file = try FileHandle(forUpdating: jsonFilePath)
//                file.write(data)
//                print("JSON data was Updated to the file \(jsonFilePath) successfully!")
//                result = jsonFilePath
//            } catch let error as NSError {
//                print("Couldn't update to file: \(jsonFilePath) \n reasons: \(error.localizedDescription)")
//            }
//            
//            
//        }
        return result
    }
    // only to be run the first time it runs
    private func loadBundleFiles() {
        let currentSession = fileNames.currentSession
        let pastSessions = fileNames.pastSessions
        let substances = fileNames.substances
        let experimentTypes = fileNames.experimentTypes
        // Build a dictionary to assing KeyPaths to each fileName
        let dataFiles = [
            currentSession
            , pastSessions
            , substances
            , experimentTypes
        ]

        for name in dataFiles {
            let fileURL = Bundle.main.url(forResource: String(name.split(separator: ".")[0]), withExtension: String(name.split(separator: ".")[1]))
            do {
                let file = try FileHandle(forReadingFrom: fileURL!)
                let data = file.readDataToEndOfFile()
                let decoder = JSONDecoder()
                decoder.dateDecodingStrategy = .iso8601
                switch name{
                case currentSession:
                    self[keyPath:\AppDelegate.currentSession] = try decoder.decode(Session.self, from: data)
                //                    print("Loading currentSession:")
                case pastSessions:
                    self[keyPath:\AppDelegate.pastSessions] = try decoder.decode([Session].self, from: data)
                case substances:
                    self[keyPath:\AppDelegate.substances] = try decoder.decode([Solution].self, from: data)
                case experimentTypes:
                    self[keyPath:\AppDelegate.experimentTypes] = try decoder.decode([Experiment].self, from: data)
                default:
                    break
                }
                
                //                    print("Successfully loaded: \(name)")
                
            } catch  {
                print("Error while trying to access the bundle JSON File for \(name)")
            }

        }
        
    }
    
}



