//
//  AddEditSolutionViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 16/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class AddEditSolutionViewController: UIViewController {

    
    var solution : Solution?
    var appDelegate : AppDelegate?
    var adding : Bool = false

    
    @IBOutlet weak var substanceNameField: UITextField!
    @IBOutlet weak var kindField: UITextField!
    @IBOutlet weak var labelConcentrationField: UITextField!
    @IBOutlet weak var acceptButton: UIBarButtonItem!
    
    @IBOutlet weak var labelConcentrationUnitsField: UITextField!
    @IBOutlet weak var factorField: UITextField!
    @IBOutlet weak var molecularWeightField: UITextField!
    @IBOutlet weak var realConcentrationField: UITextField!
    @IBOutlet weak var equivalentsField: UITextField!

    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = (UIApplication.shared.delegate as! AppDelegate)
        // Do any additional setup after loading the view.
        if self.adding {
            // if we are adding have to change the aspect of the right Nav Button
            let action = self.acceptButton.action
            let target = self.acceptButton.target
            let newButton = UIBarButtonItem(barButtonSystemItem: .save, target: target, action: action)
            self.navigationItem.rightBarButtonItem = newButton
            self.acceptButton = newButton
        } else {
            // simply update UI with model
            self.synchModelViewData()
        }
        
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        self.updateInputsFromTable()
    }
    
    func synchModelViewData(){
        self.substanceNameField.text = solution?.substance
        self.kindField.text = solution?.kind.rawValue
        self.labelConcentrationField.text = solution?.labelConcentration.description
        self.factorField.text = solution?.factor.description
        self.molecularWeightField.text = solution?.molecularWeight.description
        self.realConcentrationField.text = solution?.realConcentration?.description
        self.equivalentsField.text = solution?.equivalents.description

    }

    @IBAction func factorChanged(_ sender: UITextField) {
        self.factorField.text = self.factorField.text?.replacingOccurrences(of: ",", with: ".")
        let factor = Double(self.factorField.text!) ?? 0.0
        self.solution?.factor = factor
        self.synchModelViewData()

    }
    @IBAction func concentrationChanged(_ sender: UITextField) {
        sender.text = sender.text?.replacingOccurrences(of: ",", with: ".")

        let concValue = Double(sender.text!) ?? 0.0
        self.solution?.labelConcentration = concValue
        self.synchModelViewData()
    }
    @IBAction func substanceChanged(_ sender: UITextField) {
        sender.text = sender.text?.replacingOccurrences(of: ",", with: ".")
        let substance = sender.text
        self.solution?.substance = substance!
        self.synchModelViewData()
    }
    @IBAction func kindChanged(_ sender: UITextField) {
        let kindValue = sender.text ?? "Other"
        let kind = Kind(rawValue: kindValue) ?? Kind.other
        
        self.solution?.kind = kind
        self.synchModelViewData()
    }
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    private func updateInputsFromTable() {
        if self.substanceNameField.text?.count == 0 {
            return
        }
        self.solution?.substance = self.substanceNameField.text!
        self.solution?.kind = Kind(rawValue: self.kindField.text!)!
        self.solution?.labelConcentration = Double((self.labelConcentrationField.text?.replacingOccurrences(of: ",", with: "."))!)!
        self.solution?.factor = Double((self.factorField.text?.replacingOccurrences(of: ",", with: "."))!)!
        self.solution?.equivalents = Int(self.equivalentsField.text!)!

    }
    

    /*
    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
    }
    */
    
    // MARK: - Actions
    
    @IBAction func cancelButtonPressed(_ sender: Any) {
        dismiss(animated: true, completion: nil)
    }

    @IBAction func acceptButtonPressed(_ sender: Any) {
    }

}
