//
//  ExperimentDataInputResultsOutputViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 17/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class ExperimentDataInputResultsOutputViewController: UIViewController {

    enum constants  {
        static let presentInputDataViewSegue = "presentInputViewSegueIdentifier"
        static let startOverFromHomeIdentifier = "startOverFromStart"
    }


    var experimentSession : Session?
    var appDelegate : AppDelegate?
    
    
    
    @IBOutlet weak var outputDescriptionLabel: UILabel!
    @IBOutlet weak var inputsRepresentation: UITextView!
    @IBOutlet weak var meanLabel: UILabel!
    @IBOutlet weak var standardDeviationLabel: UILabel!
    @IBOutlet weak var outputLabel: UILabel!
    @IBOutlet weak var regressionOutputLabel: UILabel!
    
    override func viewDidLoad() {
        super.viewDidLoad()
        
        appDelegate = (UIApplication.shared.delegate as! AppDelegate)
        //Load up variables from current session
        loadFromDelegate()
        self.updateLabels()
        self.fillInputsRepresentation()

        self.outputDescriptionLabel.text = self.experimentSession?.experimentType.result?.descriptive ?? "Output"
        // Do any additional setup after loading the view.

        self.fillRegressionLabel()
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    

    // MARK: - Navigation
    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        if self.experimentSession?.series.count == 0 {
            let _ = self.experimentSession?.addSeries()
        }
        let identifier = segue.identifier!
        switch identifier {
        case constants.presentInputDataViewSegue:
            // Collect the paired variables if needed and if not, send a [Variable]
            // with the variable to fill in with data
            
            let dvc = (segue.destination as! UINavigationController).viewControllers.first! as! AddInputViewController
            
            let variableCollection = self.experimentSession?.experimentType.variables
            var varsToPass = [Variable]()
            for variable in  variableCollection! {
                if !variable.shared {
                    varsToPass.append(variable)
                }
            }
            
            dvc.variables = varsToPass
            saveToDelegate()
            
            break
        case constants.startOverFromHomeIdentifier:
            self.saveToDelegate()
            self.appDelegate?.saveState()
        default:
            break
        }
    }
    
    // MARK: - State Management
    func loadFromDelegate() {
        self.experimentSession = self.appDelegate?.currentSession ?? Session()
    }
    
    func saveToDelegate(){
        self.appDelegate?.currentSession = self.experimentSession!
        if !(self.appDelegate?.pastSessions.contains(self.experimentSession!))!{
                self.appDelegate?.pastSessions.append(self.experimentSession!)
        }
        
    }

    // MARK: - Actions
    
    // UnwindSegue we will go to when coming from adding a result
    @IBAction func unwindToExperimentDataInputOutput(sender: UIStoryboardSegue) {
        let svc = sender.source as! AddInputViewController
        let inputs = svc.inputList!
        print(inputs.description)
        if let sourceViewController = sender.source as? AddInputViewController, let inputs = sourceViewController.inputList {
            //            print("When returning to the SessionTable we got: \(expType)")
            //            print(self.sessions.count)
            
            switch inputs.count {
                
            case 1:
                self.experimentSession?.addInput(inputs.first!)
                
            case 2:
                for input in inputs {
                    self.experimentSession?.addInput(input, newSeries: false)
                }
            default:
                break
            }
            
            updateLabels()
            self.saveToDelegate()
        }
    }
    private func updateLabels() {
        getResults()
        if self.experimentSession?.series.first?.rounds.count == 0 { self.outputLabel.text = ""}
        fillInputsRepresentation()
    }

    private func fillInputsRepresentation(){
        let inputs = self.experimentSession?.series.first?.getInputs()
        var output = ""
        for input in inputs! {
            if (self.experimentSession?.experimentType.regressionType)! {
                output = output + "\(input.label): \(input.measure.value.description) \n"
            } else {
                output = output + "\(input.label): \(input.measure.description) \n"
            }
        }
        self.inputsRepresentation.text = output
    }
    
    private func getResults() {
        // don't do anything if empty inputs
        if emptyResults() {
            return
        }
        
        let experiment = self.experimentSession?.experimentType
        let regressionType = experiment?.regressionType ?? false
        var resultValues = [Double]()
        switch regressionType {
        case true:
            let series = self.experimentSession?.series.first!
            series?.regression.generateModel()
            series?.cleanResults()
            let regressionData = series?.regression.correlation.convertToFormula()
            let formula = (regressionData?.0)!
            let coefficients = (regressionData?.1)!
            var values = [Double]()
            for round in (series?.rounds)! {
                for input in round.inputs["Y"]! {
                    values.append(input.measure.value)
                }
            }
            let inputValues = values
            resultValues = (series?.getResultValuesForRegression(formula: formula, variables: coefficients, withInputs: inputValues))!
            for result in resultValues {
                series?.addResult(result, forVariable: (experiment?.result)!)
            }
            break
        default:
            // act as usual
            resultValues = self.experimentSession?.series.first?.getResultValuesForExperiment(experiment: experiment!) ?? [Double]()
        }
        
        
        if resultValues.count > 0 {
            let mean = self.experimentSession?.series.first?.getMean() ?? 0.0
            let stdDev = self.experimentSession?.series.first?.getStdDev() ?? 0.0
            let error = self.experimentSession?.series.first?.getSEM() ?? 0.0
            // https://en.wikipedia.org/wiki/Confidence_interval
            let errorMean = error * 1.96
            let meanFormater = NumberFormatter()
            if mean > 1 {
                meanFormater.maximumFractionDigits = 1
            } else {
                meanFormater.maximumFractionDigits = 4

            }
            
            let devFormater = NumberFormatter()
            if stdDev > 1 {
                devFormater.maximumFractionDigits = 1
                devFormater.minimumIntegerDigits = 1
            } else {
                devFormater.maximumFractionDigits = 4
                devFormater.minimumIntegerDigits = 1
            }
            
            let errorFormater = NumberFormatter()
            
            if errorMean > 1 {
                errorFormater.maximumFractionDigits = 1
                errorFormater.minimumIntegerDigits = 1
            } else {
                errorFormater.maximumFractionDigits = 4
                errorFormater.minimumIntegerDigits = 1

            }
            
            let meanOutput = meanFormater.string(from: NSNumber(value:mean))!
            let stdDevOutput = devFormater.string(from: NSNumber(value:stdDev))!
            let errorOutput = errorFormater.string(from: NSNumber(value:errorMean))!

            self.meanLabel.text = "Mean: \(meanOutput)"
            self.standardDeviationLabel.text = "Std dev: \(stdDevOutput)"
            self.outputLabel.text = "Output: \(meanOutput) ± \(errorOutput)%"

        } else {
            self.outputLabel.text = ""
        }
        self.fillRegressionLabel()

    }
    func fillRegressionLabel() {
        if (self.experimentSession?.experimentType.regressionType)! {
            // if we have a regression type, enable the regression label
            let regression = self.experimentSession?.series.first?.regression
            let regressionOutput = (regression?.correlation.description)!
            let coefFormatter = NumberFormatter()
            coefFormatter.minimumIntegerDigits = 1
            coefFormatter.maximumFractionDigits = 4
            coefFormatter.minimumFractionDigits = 4
            let rCoeff = (coefFormatter.string(from: NSNumber(value:(regression?.correlation.rCoefficient)!)))!
            self.regressionOutputLabel.text = "\(regressionOutput) (R=\(rCoeff))"
        }

    }
    
    private func emptyResults() -> Bool {
        return self.experimentSession?.series[(self.experimentSession?.currentSerie)!].rounds.count == 0
    }
}
