//
//  ExperimentSharedInputsTableViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 17/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class ExperimentSharedInputsTableViewController: UITableViewController {

    var variables = [Variable]()
    var session = Session()
    var appDelegate : AppDelegate?
    var filledInputs = [Input]()

    
    enum constants  {
        static let cellIdentifier = "SharedInputTableViewCell"
        static let goToDataInputSegue = "goToDataInputSegueIdentifier"
    }

    override func viewDidLoad() {
        super.viewDidLoad()

        appDelegate = (UIApplication.shared.delegate as! AppDelegate)

        self.loadFromDelegate()
        
        
        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem
        
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .save, target: self, action: #selector(ExperimentSharedInputsTableViewController.goToDataInputViewController))

    }
    

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - State Management
    func loadFromDelegate() {
        self.variables = (self.appDelegate?.currentSession.experimentType.variables) ?? [Variable]()
        for (index, variable) in self.variables.enumerated() {
            if !variable.shared {
                self.variables.remove(at: index)
            }
        }
        
        self.session = (self.appDelegate?.currentSession) ?? Session()

    }
    
    func saveToDelegate(){
        self.appDelegate?.currentSession = self.session
    }


    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        return 2
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        var result = 1
        switch section {
        case 0:
            break
        case 1:
            result = self.variables.count
        default:
            break
        }
        return result
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: constants.cellIdentifier, for: indexPath) as! SharedInputTableViewCell
        let section = indexPath.section
        cell.inputValueField.clearsOnBeginEditing = true
        switch section {
        case 0:
            // return the experiment description cell
            cell.variableNameLabel.text = "Experiment name"
            cell.inputValueField.keyboardType = .default
            cell.inputValueField.text = self.session.description
            cell.inputUnitsLabel.text = ""
            
            
        default:
            // Configure the cell...
            let variable = variables[indexPath.row]
            cell.variableNameLabel.text = variable.descriptive
            let inputValue = (self.session.series[self.session.currentSerie].getShared().filter({$0.label == variable.label}).first)?.measure.value.description ?? 0.0.description
            
            
            cell.inputValueField.keyboardType = .decimalPad
            cell.inputValueField.text = inputValue
            switch variable.dimension {
            case .mass:
                cell.inputUnitsLabel.text = UnitMass.grams.symbol
            case .volume:
                cell.inputUnitsLabel.text = UnitVolume.milliliters.symbol
                
            case .concentration:
                cell.inputUnitsLabel.text = UnitConcentrationMass.molar.symbol
                
                
            case .regression:
                cell.inputUnitsLabel.text = ""

            case .percentage:
                cell.inputUnitsLabel.text = "%"
}
            
        }

        return cell
    }

    /*
    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    */

    /*
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }
    */

    /*
    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {

    }
    */

    /*
    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the item to be re-orderable.
        return true
    }
    */

    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        let identifier = segue.identifier!
        // before going to the next view controller, save current session and add current session to past sessions in delegate
        
        switch identifier {
        case constants.goToDataInputSegue:
            self.appDelegate?.currentSession = self.session
            let mySession = self.session
            let pastSessions = self.appDelegate?.pastSessions ?? [Session]()
            let index = pastSessions.index(of: mySession) ?? -1
            
            if index < 0 {
                self.appDelegate?.pastSessions.append(self.session)
            }
            
default:
            break
        }
        
    }

    
    // MARK: - Actions
    
    @objc func goToDataInputViewController() {
        // update all the fields in the table
        self.updateInputsFromTable()
        
        
        self.session.cleanShared()
        for input in self.filledInputs {
            self.session.addInput(input)
        }
        self.filledInputs = [Input]()
        saveToDelegate()
        self.performSegue(withIdentifier: constants.goToDataInputSegue, sender: self)
        
    }
    
    @IBAction func fieldChanged(_ sender: UITextField, forEvent event: UIEvent) {
        
        let cell = sender.superview?.superview as! SharedInputTableViewCell
        
        let indexPath = tableView.indexPath(for: cell)!
        
        switch indexPath.section {
        case 0:
            self.session.description = sender.text!
        case 1:
            //value for one of the variables changed
//            let variable = self.variables[indexPath.row]
//            let input = Input(fromVariable: variable, withValue: Double(sender.text!)!)
//            self.filledInputs.append(input)
            break
            
        default:
            break
        }
        
    }
    
    private func updateInputsFromTable() {
        
        for (index, variable) in self.variables.enumerated() {
            // for each variable, find out the value in the cell for that row and save it as input
            let section = 1
            let row = index
            let indexPath = IndexPath(row: row, section: section)
            let cell = tableView.cellForRow(at: indexPath) as! SharedInputTableViewCell
            // be carefull with the comma
            cell.inputValueField.text = cell.inputValueField.text?.replacingOccurrences(of: ",", with: ".")

            let value = Double(cell.inputValueField.text!)!
            let input = Input(fromVariable: variable, withValue: value)
            self.filledInputs.append(input)
        }
        
    }
    
    
}
