//
//  ExperimentsTableViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 10/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class ExperimentsTableViewController: UITableViewController , UISearchControllerDelegate{
    enum constants : String {
        case cellIdentifier = "ExperimentSessionsTableViewCellIdentifier"
        case presentInputsSegueIdentifier = "presentSharedInputsForExperiment"
        case presentExperimetTypesSegueIdentifier = "showExperimentTypesSegue"
        case goToRegressionInputSegue = "presentRegressiondataInputSegueIdentifier"
    }
    
    var appDelegate : AppDelegate?

    var currentSession = Session()
    var sessions = [Session]()

    override func viewDidLoad() {
        super.viewDidLoad()

        // Uncomment the following line to preserve selection between presentations
         self.clearsSelectionOnViewWillAppear = false
        appDelegate = (UIApplication.shared.delegate as! AppDelegate)
        loadFromDelegate()
        
        
        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem
        
        // adding a navBar button programmatically
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(ExperimentsTableViewController.goToExperimentTypesController))
        
    }
    override func viewWillAppear(_ animated: Bool) {
        //print("view will appear")
        loadFromDelegate()
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        saveToDelegate()
    }
    

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - State Management
    func loadFromDelegate() {
        self.sessions = self.appDelegate?.pastSessions ?? [Session]()
        self.currentSession = self.appDelegate?.currentSession ?? Session()
        tableView.reloadData()
    }
    
    func saveToDelegate(){
        self.appDelegate?.pastSessions = self.sessions
        self.appDelegate?.currentSession = self.currentSession
    }

    
    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        return sessions.count
    }

    
    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: constants.cellIdentifier.rawValue, for: indexPath) as! ExperimentSessionsTableViewCell

        // Configure the cell...
        let session = self.sessions[indexPath.row]
        cell.nameLabel.text = session.description
        let formater = DateFormatter()
        formater.dateStyle = .medium
        formater.timeStyle = .none
        let prettyDate = formater.string(from: session.startDate)
//        print(prettyDate)
        cell.startDateLabel.text = prettyDate
        let seriesCount = session.series.count
        cell.seriesLabel.text = "\(seriesCount) rounds"

        return cell
    }
    

    /*
    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }
    */

    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
            let newSession = Session()
            self.sessions.append(newSession)
        }    
    }
    //https://stackoverflow.com/a/32586617/2772618
    // Adding custom swipe behaviours
    override func tableView(_ tableView: UITableView, editActionsForRowAt: IndexPath) -> [UITableViewRowAction]? {
        let duplicate = UITableViewRowAction(style: .normal, title: "New like this") { action, index in
            switch index.row {
            case 0..<self.sessions.count:
                let newSession = self.sessions[index.row].copySessionToNew()
                newSession.description = "\(newSession.description) copy"
                
                self.sessions.insert(newSession, at: index.row+1)
                //            tableView.reloadData()
                tableView.insertRows(at: [index], with: .automatic)
                
                self.appDelegate?.pastSessions = self.sessions
                self.tableView.reloadData()
                
            default:
                print("Carefull there, we got invalid index.row!")
                break
            }
        }
        duplicate.backgroundColor = UIColor(red: 0, green: 0.7098, blue: 0.0353, alpha: 1.0)
        
        let delete = UITableViewRowAction(style: .default, title: "Delete") { action, index in
            switch index.row {
            case 0..<self.sessions.count:
                self.sessions.remove(at: index.row)
                tableView.deleteRows(at: [index], with: .automatic)
                self.appDelegate?.pastSessions = self.sessions

            default:
                print("Carefull there, we got invalid index.row!")
                break
            }
        }
        delete.backgroundColor = .orange
        // Enabling this creates a nasty crahsing bug, watch it!
//        tableView.reloadData()
        return [duplicate, delete]
    }


    /*
    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {

    }
    */

    /*
    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the item to be re-orderable.
        return true
    }
    */
    
    // MARK: - Actions
    
    @IBAction func unwindToSessions(sender: UIStoryboardSegue) {
        if let sourceViewController = sender.source as? ExperimentTypeTableViewController, let expType = sourceViewController.selected {
//            print("When returning to the SessionTable we got: \(expType)")
//            print(self.sessions.count)
            let newSession = Session.newSessionWithExperimentType(expType)
            self.currentSession = newSession
            self.sessions.append(newSession)
            let indexPath = IndexPath(row: self.sessions.count - 1, section: 0)
            tableView.insertRows(at: [indexPath], with: .automatic)

            self.saveToDelegate()
            
        }
    }
    


    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        self.appDelegate?.pastSessions = self.sessions
        let segueIdentifier = segue.identifier ?? ""
        switch segueIdentifier {
        case constants.presentExperimetTypesSegueIdentifier.rawValue:
            let navigationViewController = segue.destination as! UINavigationController
            let dvc = navigationViewController.viewControllers[0] as! ExperimentTypeTableViewController
            let expTypes = self.appDelegate?.experimentTypes ?? [Experiment]()
            dvc.experiments = expTypes
            
            
        case constants.presentInputsSegueIdentifier.rawValue:
//            let dvc = segue.destination as! ExperimentSharedInputsPresentationViewController
            if self.currentSession.experimentType.regressionType == true {
                self.performSegue(withIdentifier: constants.goToRegressionInputSegue.rawValue, sender: self)
            }
            let dvc = segue.destination as! ExperimentSharedInputsTableViewController
            dvc.appDelegate = self.appDelegate
            
            // we know the sender is the tableviewCell that contains the session we want to continue
            let cell = sender as! ExperimentSessionsTableViewCell
            let indexPath = tableView.indexPath(for: cell)
            
            self.currentSession = self.sessions[(indexPath?.row)!]
            
            self.saveToDelegate()
            
        case constants.goToRegressionInputSegue.rawValue:
            
            let nvc = segue.destination as! UINavigationController
            let vc = nvc.viewControllers.first! as! RegressionTableViewController
            //we have the destination view controller regression view controller
            vc.data = self.currentSession.series.first!.regression.points
        default:
            break
        }
    }

    
    
    @objc func goToExperimentTypesController(){
//        print("Going to ExperimentTypeController!")
        performSegue(withIdentifier: constants.presentExperimetTypesSegueIdentifier.rawValue, sender: self)
        
    }
    
    public func addSession(_ session: Session){
        self.sessions.append(session)
    }

}
