//
//  ViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 22/10/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit
import AVFoundation


class HomeScreenViewController: UIViewController, QRCodeReaderViewControllerDelegate {
    
    @IBOutlet weak var appLogo: UIImageView!
    @IBOutlet weak var newExperimentButton: UIButton!
    @IBOutlet weak var substancesButton: UIButton!
    @IBOutlet weak var readQRButton: UIButton!


    enum constants : String {
        case presentExperimentsSegue = "presentExperiments"
        case presentSubstancesSegue = "presentSolutions"
        case presentQRReaderSegue = "presentQRReader"
        case presentQRCodePresenter = "presentQRCodeGeneratedSegue"
        
    }

    var currentExperiment : Experiment?
    var appDelegate : AppDelegate?
    var message = ""

    
    override func viewDidLoad() {
        super.viewDidLoad()
        appDelegate = (UIApplication.shared.delegate as! AppDelegate)
        appDelegate?.loadState()
//        self.appDelegate?.experimentTypes =  self.populateExperiments()
        
//        let session = self.appDelegate?.currentSession
//        let sessionString = (session?.exportString())!
//
//        let message = self.prepareQRPayloadFor(item: session!)
        
        
//        print("\n\nString encoded: \(message.exportString())")
        

        
//        session?.decodeFrom(string: sessionString)
        
        
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

     // MARK: - Navigation
     
     // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        appDelegate?.saveState()
        
        let segueIdentifier = segue.identifier ?? ""
        switch segueIdentifier {
        case constants.presentExperimentsSegue.rawValue:
            let dvc = segue.destination as? ExperimentsTableViewController
            dvc?.sessions = self.appDelegate?.pastSessions ?? [Session]()
            dvc?.appDelegate = self.appDelegate
            break
        case constants.presentSubstancesSegue.rawValue:
            let dvc = segue.destination as? SolutionsTableViewController
            dvc?.solutions = self.appDelegate?.substances ?? [Solution]()
            //dvc?.appDelegate = self.appDelegate
            
        case constants.presentQRCodePresenter.rawValue:
            let nvc = segue.destination as! UINavigationController
            let dvc = nvc.viewControllers.first! as! QRViewerViewController
            
            self.message = self.prepareQRPayloadFor(item: (self.appDelegate?.substances.first!)).exportString()
            
//            dvc.messageString = (self.appDelegate?.currentSession.exportString())!
            dvc.messageString = self.message
        default:
            break
        }
    }

    /*
 "π"
 "e"
 "√"
 "×"
 "−"
 "÷"
 "plus"
 "="
 "c"
 */
 

    // MARK: - Json Payload Handling
    func prepareQRPayloadFor<T: Encodable>(item: T) -> QRPayloadContainer {
        switch item.self {
        case is Experiment:
            let payloadString = (item.exportString())
            
            return QRPayloadContainer(kind: .experimentType, payload: payloadString.data(using: .utf8)!)
            
        case is Solution:
            let payloadString = (item.exportString())
            
            return QRPayloadContainer(kind: .solution, payload: payloadString.data(using: .utf8)!)
            
        default:
            let payloadString = (item.exportString())
            
            return QRPayloadContainer(kind: .other, payload: payloadString.data(using: .utf8)!)
            
        }
    }

    // MARK: - Testing functions
    func populateExperiments() -> [Experiment]{
        let exp1 = Experiment()
        exp1.name = "Experiment 1"
        exp1.description = "Very interesting experiment, seriosuly, try it!"
        exp1.formula = Experiment.stepsFromFormula(formula: "initialMass × initialVolume plus 25 =")
        let var1 = Variable(label: "initialMass", dim: .mass, shared: true, kind: .other)
        let var2 = Variable(label: "initialVolume", dim: .volume, shared: true, kind: .other)
        let var3 = Variable(label: "volumeUsed", dim: .volume, shared: false, kind: .other)
        exp1.variables = [var1, var2, var3]
        
        return [exp1]
    }
    
    func populateSubstances() -> [Solution]{
        let solution1 = Solution(substance: "NaOH", atMolarity: 0.1, factor: 1.002, equivalence: 1, MW:nil, kind: .base)
        let solution2 = Solution(substance: "HCl", atMolarity: 0.01, factor: 0.998, equivalence: 1, MW:nil, kind: .acid)
        let solution3 = Solution(substance: "H2SO4", atMolarity: 0.01, factor: 1.001, equivalence: 2, MW:nil, kind: .acid)
        let solution4 = Solution(substance: "HNO3", atMolarity: 0.01, factor: 0.989, equivalence: 1, MW:nil, kind: .acid)
        let solution5 = Solution(substance: "KOH", atMolarity: 0.01, factor: 0.989, equivalence: 1, MW:nil, kind: .base) 
        
        
        
        return [solution1, solution2, solution3, solution4, solution5]
    }
    
    func populatePastSessions() -> [Session] {
        let session1 = Session()
        session1.experimentType = appDelegate?.experimentTypes[0] ?? Experiment()
        session1.startDate = Date(timeIntervalSinceNow: -118800.0)
        let session2 = Session()
        session2.experimentType = appDelegate?.experimentTypes[0] ?? Experiment()
        session2.startDate = Date()
        
        return [session1, session2]

    }
    
    
    //MARK: -
    // MARK: QR Reader implementation - Vars
    lazy var reader: QRCodeReader = QRCodeReader()
    lazy var readerVC: QRCodeReaderViewController = {
        let builder = QRCodeReaderViewControllerBuilder {
            $0.reader = QRCodeReader(metadataObjectTypes: [AVMetadataObject.ObjectType.qr], captureDevicePosition: .back)
            $0.showTorchButton = true
        }
        
        return QRCodeReaderViewController(builder: builder)
    }()

    // MARK: QR Reader implementation - Actions
    
    private func checkScanPermissions() -> Bool {
        do {
            return try QRCodeReader.supportsMetadataObjectTypes()
        } catch let error as NSError {
            let alert: UIAlertController
            
            switch error.code {
            case -11852:
                alert = UIAlertController(title: "Error", message: "This app is not authorized to use Back Camera.", preferredStyle: .alert)
                
                alert.addAction(UIAlertAction(title: "Setting", style: .default, handler: { (_) in
                    DispatchQueue.main.async {
                        if let settingsURL = URL(string: UIApplicationOpenSettingsURLString) {
//                            UIApplication.shared.openURL(settingsURL)
                            UIApplication.shared.open(settingsURL,  completionHandler: nil)
                        }
                    }
                }))
                
                alert.addAction(UIAlertAction(title: "Cancel", style: .cancel, handler: nil))
            default:
                alert = UIAlertController(title: "Error", message: "Reader not supported by the current device", preferredStyle: .alert)
                alert.addAction(UIAlertAction(title: "OK", style: .cancel, handler: nil))
            }
            
            present(alert, animated: true, completion: nil)
            
            return false
        }
    }
    
    @IBAction func scanInModalAction(_ sender: AnyObject) {
        guard checkScanPermissions() else { return }
        
        readerVC.modalPresentationStyle = .formSheet
        readerVC.delegate               = self
        
        readerVC.completionBlock = { (result: QRCodeReaderResult?) in
            if let result = result {
                print("Completion with result: \(result.value) of type \(result.metadataType)")
            }
        }
        
        present(readerVC, animated: true, completion: nil)
    }

    // MARK: QRCodeReader Delegate Methods
    
    func reader(_ reader: QRCodeReaderViewController, didScanResult result: QRCodeReaderResult) {
        reader.stopScanning()
        // Dismiss message
        dismiss(animated: true, completion: nil)
//        dismiss(animated: true) { [weak self] in
//            let alert = UIAlertController(
//                title: "QRCodeReader",
//                message: String (format:"%@ (of type %@)", result.value, result.metadataType),
//                preferredStyle: .alert
//            )
//            alert.addAction(UIAlertAction(title: "OK", style: .cancel, handler: nil))
//
//            self?.present(alert, animated: true, completion: nil)
//        }
    
        // read the QR info, determine the kind of payload codified and create the relevant entity
        // Go to the relevant VC to simulate the addition of said element (ExperimentType/Solution/Serie??)
        
        let decoder = JSONDecoder()
        decoder.dateDecodingStrategy = .iso8601
        var payloadType : QRPayloadKind = .other
        var payload = Data()
        
        
        
        do {
            let qrResult = try decoder.decode(QRPayloadContainer.self, from: result.value.data(using: .utf8)!)
            payloadType = qrResult.kind
            // decompress manually the payload that was compressed to reduce QR Size
            payload = qrResult.payload.decompress(withAlgorithm: .LZFSE)!
//            payload = qrResult.payload
            
        } catch(let error) {
            print("Failure decoding payload \(error)")
        }
        
        switch payloadType {
        case .experimentType:
            // generate a experiment from payload and move on
            do {
                let experiment = try decoder.decode(Experiment.self, from: payload)
                // directly to the point
                self.appDelegate?.experimentTypes.append(experiment)
                
                
                // ideally go to experimentType add/edit, fill up UI and present the user the option to add content
                
            } catch {
                print("unable to decode data for experiment type")
            }
            
        case .solution:
            // generate a experiment from payload and move on
            do {
                let solution = try decoder.decode(Solution.self, from: payload)
                // directly to the point
                self.appDelegate?.substances.append(solution)
                // ideally go to experimentType add/edit, fill up UI and present the user the option to add content
//                let vc = AddEditSolutionViewController()
//                vc.solution = solution
//                vc.adding = true
//                self.present(vc, animated: true, completion: nil)
                
                
            } catch {
                print("unable to decode data for experiment type")
            }

default:
            break
        }
        
        
    }
    
    func reader(_ reader: QRCodeReaderViewController, didSwitchCamera newCaptureDevice: AVCaptureDeviceInput) {
        print("Switching capturing to: \(newCaptureDevice.device.localizedName)")
    }
    
    func readerDidCancel(_ reader: QRCodeReaderViewController) {
        reader.stopScanning()
        
        dismiss(animated: true, completion: nil)
    }
}

