//
//  RegressionTableViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 30/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class RegressionTableViewController: UITableViewController {

    var data = [Datapoint]()
    var regressionModel = Regression()
    var appDelegate : AppDelegate?
    
    
    enum constants : String {
        case cellIdentifier = "regressionInputDataCellIdentifier"
        case xFieldIdentifier = "xValueField"
        case yFieldIdentifier = "yValueField"
        case finishInputAndGoToResults = "finishInputAndGoToResults"
        case slope = "slope"
        case intercept = "intercept"
    }

    
    
    override func viewDidLoad() {
        super.viewDidLoad()

        appDelegate = (UIApplication.shared.delegate as! AppDelegate)

        // Uncomment the following line to preserve selection between presentations
        // self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem
//        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action: #selector(RegressionTableViewController.calculateRegression))
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .done, target: self, action:#selector(RegressionTableViewController.finishAndGoToInputs))
        if self.data.count == 0 {self.addDataRow()}
    }

    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }

    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        return data.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: constants.cellIdentifier.rawValue, for: indexPath) as! RegressionTableViewCell

        // Configure the cell...
        cell.xField.text = "\(self.data[indexPath.row].x)"
        cell.yField.text = "\(self.data[indexPath.row].y)"
        
        
        return cell
    }

    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }


    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            self.data.remove(at: indexPath.row)
            tableView.reloadData()
//            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
            let newData = Datapoint()
            let x = 0.0
            let y = 0.0
            newData.x = x
            newData.y = y
            data.append(newData)
        }
    }

    /*
    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {

    }
    */

    /*
    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the item to be re-orderable.
        return true
    }
    */

    // MARK: - Navigation
    
    @objc func finishAndGoToInputs(){
        self.updateDataFromTable()
        self.calculateRegression()
        self.performSegue(withIdentifier: constants.finishInputAndGoToResults.rawValue, sender: self)
        
    }

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        
        let identifier = segue.identifier!
        
        switch identifier {
        case constants.finishInputAndGoToResults.rawValue:
            //we are in business
//            let vc = segue.destination as! ExperimentDataInputResultsOutputViewController
            self.synchTableValuesToModel()
            self.calculateRegression()
            self.appDelegate?.currentSession.series.first?.regression = self.regressionModel
            
//            vc.experimentSession?.series.first?.regression = self.regressionModel
            
            
            
        default:
            break
        }
    }
    // We need to ensure that all values in the table are correctly formated and
    // stored in the data used for the regression
    func updateDataFromTable() {
        for (index, point) in self.data.enumerated() {
            let section = 0
            let row = index
            let indexPath = IndexPath(row: row, section: section)
            let cell = tableView.cellForRow(at: indexPath) as! RegressionTableViewCell
            cell.xField.text = cell.xField.text?.replacingOccurrences(of: ",", with: ".")
            cell.yField.text = cell.yField.text?.replacingOccurrences(of: ",", with: ".")
            let xValue = Double(cell.xField.text!)!
            let yValue = Double(cell.yField.text!)!
            
            let newPoint = Datapoint()
            newPoint.x = xValue
            newPoint.y = yValue
            
            data[index] = newPoint

        }
    }

    
    // MARK: - Actions
    @IBAction func valueChanged(_ sender: UITextField) {
        if let value = Double(sender.text!.replacingOccurrences(of: ",", with: ".")) {
            print(value)
            let field = sender
            let contentView = field.superview!
            let cell = contentView.superview! as! RegressionTableViewCell
            let indexRow = tableView.indexPath(for: cell)
            let dataRow = (indexRow?.row)!
            //            print("\(indexRow?.row)")
            switch field.restorationIdentifier! {
            case constants.xFieldIdentifier.rawValue:
                data[dataRow].x = value
            case constants.yFieldIdentifier.rawValue:
                data[dataRow].y = value
            default:
                break
            }
            
        }
        
    }
    
   @objc func addDataRow(){
        let newRow = Datapoint()
        newRow.x = 0.0
        newRow.y = 0.0
        data.append(newRow)
        self.tableView.reloadData()
    }
    
    @IBAction func insertValue(_ sender: UIButton) {
        let button = sender
        let contentView = button.superview!
        let cell = contentView.superview! as! RegressionTableViewCell
        let xValue = Double(cell.xField.text!.replacingOccurrences(of: ",", with: ".")) ?? 0.0
        let yValue = Double(cell.yField.text!.replacingOccurrences(of: ",", with: ".")) ?? 0.0
        let indexRow = tableView.indexPath(for: cell)
        let dataRow = (indexRow?.row)!
        //            print("\(indexRow?.row)")
        data[dataRow].x = xValue
        data[dataRow].y = yValue
        self.addDataRow()
        
    }
    func synchTableValuesToModel(){
        let cells = tableView.visibleCells as! [RegressionTableViewCell]
        
        for cell in cells {
            let xValue = Double(cell.xField.text!) ?? 0.0
            let yValue = Double(cell.yField.text!) ?? 0.0
            let indexRow = tableView.indexPath(for: cell)
            let dataRow = (indexRow?.row)!
            data[dataRow].x = xValue
            data[dataRow].y = yValue
        }
    }
    
    func calculateRegression() {
        
        synchTableValuesToModel()
        self.regressionModel.points = self.data
        self.regressionModel.generateModel(data: self.data)
        
        print("Regression created. m= \(self.regressionModel.correlation.components[constants.slope.rawValue]!) , c= \(self.regressionModel.correlation.components[constants.intercept.rawValue]!) , r= \(self.regressionModel.correlation.rCoefficient)")
        
    }
    
}
