//
//  SolutionsTableViewController.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 16/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

class SolutionsTableViewController: UITableViewController {

    // MARK: Model container
    
    var solutions = [Solution]()
    var appDelegate : AppDelegate?
    var selected : Solution?
    

    enum constants : String {
        case cellIdentifier = "SolutionsTableViewCellIdentifier"
        case addSolution = "addSolutionSegue"
        case editSolution = "editSolutionSegue"
        case presentQRIdentifier = "PresentQRViewControllerIdentifier"
    }

    
    
    
    override func viewDidLoad() {
        super.viewDidLoad()

        appDelegate = (UIApplication.shared.delegate as! AppDelegate)
        self.loadFromDelegate()
        print("Substances loaded from delegate: \(self.solutions.count)")
        
        // Uncomment the following line to preserve selection between presentations
        self.clearsSelectionOnViewWillAppear = false

        // Uncomment the following line to display an Edit button in the navigation bar for this view controller.
        // self.navigationItem.rightBarButtonItem = self.editButtonItem
        
        self.navigationItem.rightBarButtonItem = UIBarButtonItem(barButtonSystemItem: .add, target: self, action: #selector(SolutionsTableViewController.goToAddSolution))

        
    }
    override func viewWillAppear(_ animated: Bool) {
        //print("view will appear")
        self.loadFromDelegate()
        
    }
    
    override func viewWillDisappear(_ animated: Bool) {
        self.saveToDelegate()
        print("solutions table dissapearing, saved substances")
    }


    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
        // Dispose of any resources that can be recreated.
    }
    
    // MARK: - Model handling
    func loadFromDelegate() {
        self.solutions = self.appDelegate?.substances ?? [Solution]()
    }
    
    func saveToDelegate(){
        self.appDelegate?.substances = self.solutions
    }
    // MARK: - Actions
    @IBAction func unwindToSolutions(sender: UIStoryboardSegue) {
        
        // we have to update the self.solutions with the new value or update the previous one
        let sender = sender.source as! AddEditSolutionViewController
        if sender.adding {
            self.solutions.append(sender.solution!)
        } else {
            let indexPath = tableView.indexPathForSelectedRow!
            self.solutions[indexPath.row] = sender.solution!
        }
        //actually save them to the appDelegate for persistence
        
        self.saveToDelegate()
        tableView.reloadData()
        
    }


    // MARK: - Table view data source

    override func numberOfSections(in tableView: UITableView) -> Int {
        // #warning Incomplete implementation, return the number of sections
        return 1
    }

    override func tableView(_ tableView: UITableView, numberOfRowsInSection section: Int) -> Int {
        // #warning Incomplete implementation, return the number of rows
        return self.solutions.count
    }

    override func tableView(_ tableView: UITableView, cellForRowAt indexPath: IndexPath) -> UITableViewCell {
        let cell = tableView.dequeueReusableCell(withIdentifier: constants.cellIdentifier.rawValue, for: indexPath) as! SolutionsTableViewCell

        // Configure the cell...
        let solution = self.solutions[indexPath.row]
        cell.substanceLabel.text = solution.substance
        cell.detailsLabel.text = "\(solution.labelConcentration) M (f:\(solution.factor))"
        
        return cell
    }
    
    override func tableView(_ tableView: UITableView, editActionsForRowAt: IndexPath) -> [UITableViewRowAction]? {
        
//        print("We receive the indexPath -> section=\(editActionsForRowAt.section), row=\(editActionsForRowAt.row)")
        
        
        let duplicate = UITableViewRowAction(style: .default, title: "Duplicate") { action, index in
            //            print("NewLikeThis pressed")
            switch index.row {
            case 0..<self.solutions.count:
                let newSolution = self.solutions[index.row].copySolutionToNew()
                
                self.solutions.insert(newSolution, at: index.row)
                
                tableView.insertRows(at: [index], with: .automatic)
                
                self.appDelegate?.substances = self.solutions

            default:
                print("Carefull there, we got invalid index.row!")
                break
            }
            
        }
        duplicate.backgroundColor = UIColor(red: 0, green: 0.7098, blue: 0.0353, alpha: 1.0)
        
        let exportQR = UITableViewRowAction(style: .default, title: "ExportQR") { action, index in
            self.selected = self.solutions[index.row]
            self.performSegue(withIdentifier: constants.presentQRIdentifier.rawValue, sender: self)
            
        }
        exportQR.backgroundColor = .blue
        

        let delete = UITableViewRowAction(style: .default, title: "Delete") { action, index in
            //            print("Delete pressed")
            self.solutions.remove(at: index.row)
            tableView.deleteRows(at: [index], with: .automatic)
            self.appDelegate?.substances = self.solutions
        }
        delete.backgroundColor = .orange
        
        
//        tableView.reloadData()
        return [duplicate, exportQR, delete]
        
    }

    // Override to support conditional editing of the table view.
    override func tableView(_ tableView: UITableView, canEditRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the specified item to be editable.
        return true
    }

    /*
    // Override to support editing the table view.
    override func tableView(_ tableView: UITableView, commit editingStyle: UITableViewCellEditingStyle, forRowAt indexPath: IndexPath) {
        if editingStyle == .delete {
            // Delete the row from the data source
            tableView.deleteRows(at: [indexPath], with: .fade)
        } else if editingStyle == .insert {
            // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
        }    
    }
    */

    /*
    // Override to support rearranging the table view.
    override func tableView(_ tableView: UITableView, moveRowAt fromIndexPath: IndexPath, to: IndexPath) {

    }
    */

    /*
    // Override to support conditional rearranging of the table view.
    override func tableView(_ tableView: UITableView, canMoveRowAt indexPath: IndexPath) -> Bool {
        // Return false if you do not want the item to be re-orderable.
        return true
    }
    */

    // MARK: - Navigation

    // In a storyboard-based application, you will often want to do a little preparation before navigation
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        // Get the new view controller using segue.destinationViewController.
        // Pass the selected object to the new view controller.
        
        let identifier = segue.identifier ?? ""
        
        switch identifier {
        case constants.addSolution.rawValue:
            let navigationCotroller = segue.destination as? UINavigationController
            let dvc = navigationCotroller?.viewControllers[0] as? AddEditSolutionViewController
            dvc?.adding = true
            dvc?.solution = Solution()
            
        case constants.editSolution.rawValue:
            // sender is the cell that has been clicked, so:
            let cell = sender as! SolutionsTableViewCell
            let indexPath = tableView.indexPath(for: cell)
            let solution = self.solutions[(indexPath?.row)!]
            let navigationCotroller = segue.destination as? UINavigationController
            let dvc = navigationCotroller?.viewControllers[0] as? AddEditSolutionViewController
            dvc?.solution = solution
        case constants.presentQRIdentifier.rawValue:
            let nvc = segue.destination as! UINavigationController
            let dvc = nvc.viewControllers.first! as! QRViewerViewController
            
            let message = self.selected.prepareQRPayloadFor(item: self.selected!)
            
            //            dvc.messageString = (self.appDelegate?.currentSession.exportString())!
            dvc.messageString = message.exportString()
            break
        default:
            break
        }
    }
    
    @objc func goToAddSolution(){
        //        print("Going to ExperimentTypeController!")
        performSegue(withIdentifier: "addSolutionSegue", sender: self)
    }
    



}
