//
//  MeasurementHelperTypes.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 11/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation

public enum SupportedDimensions : String , Codable{
    case mass = "mass"
    case volume = "volume"
    case concentration = "concentration"
    case regression = "regression"
    case percentage = "percentage"
}

public enum Kind : String , Codable {
    case acid = "acid"
    case base = "base"
    case other = "other"
}

public enum QRPayloadKind : Int, Codable{
    case session = 0
    case solution
    case experimentType
    case sessionCollection
    case solutionCollection
    case experimentTypeCollection
    case other
    
}
public enum ComponentLabels  {
    static let slope = "slope"
    static let intercept = "intercept"
}



public class QRPayloadContainer : Codable {
    var kind : QRPayloadKind
    var payload : Data
    
    init(kind: QRPayloadKind, payload: Data){
        self.kind = kind
        self.payload = payload
    }
    
    public func exportString() -> String {
        
        let encoder = JSONEncoder()
        encoder.dateEncodingStrategy = .iso8601
        var data = Data()
        do {
            data = try encoder.encode(self)
        } catch  {
            print("Failure encoding \(String(describing:self))")
        }
        let output = String(data:data, encoding: .utf8) ?? ""
        
        return output
        
    }
    

}

extension Encodable {
    public func exportString() -> String {
        
        let encoder = JSONEncoder()
        encoder.dateEncodingStrategy = .iso8601
        var data = Data()
        do {
            data = try encoder.encode(self)
        } catch  {
            print("Failure encoding \(String(describing:self))")
        }
        let output = String(data:data, encoding: .utf8) ?? ""
        
        return output
        
    }
    func prepareQRPayloadFor<T: Encodable>(item: T) -> QRPayloadContainer {
        
        let payloadString = (item.exportString())
        let data = payloadString.data(using: .utf8)!
        let compressedData = data.compress(withAlgorithm: .LZFSE)!

        switch item.self {
        case is Experiment:
            
            return QRPayloadContainer(kind: .experimentType, payload: compressedData)
            
        case is Solution:

            return QRPayloadContainer(kind: .solution, payload: compressedData)
            
        default:

            return QRPayloadContainer(kind: .other, payload: compressedData)
            
        }
    }


}

public func ==<K, V: Hashable>(lhs: [K: V], rhs: [K: V] ) -> Bool {
    return (lhs as NSDictionary).isEqual(to: rhs)
}

