//
//  MolarityUnitExtension.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 17/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

// extracted from https://davidcodes.wordpress.com/2016/10/25/custom-scientific-measurements-using-the-new-nsmeasurement-class/

import Foundation


precedencegroup ExponentiativePrecedence {
    
    associativity: right
    
    higherThan: MultiplicationPrecedence
    
}

// Put this at file level anywhere in your project

infix operator ^^: ExponentiativePrecedence

public func ^^ (radix: Double, power: Double) -> Double {
    
    return pow((radix), (power))
    
}


public extension UnitConcentrationMass {
    
    
    
     static let molar = UnitConcentrationMass(symbol: "M", converter: UnitConverterLinear(coefficient: 1))
    
     static let millimolar = UnitConcentrationMass(symbol: "mM", converter: UnitConverterLinear(coefficient: 10 ^^ -3))
    
     static let micromolar = UnitConcentrationMass(symbol: "µM", converter: UnitConverterLinear(coefficient: 10 ^^ -6))
    
     static let nanomolar = UnitConcentrationMass(symbol: "nM", converter: UnitConverterLinear(coefficient: 10 ^^ -9))
    
     static let picomolar = UnitConcentrationMass(symbol: "pM", converter: UnitConverterLinear(coefficient: 10 ^^ -12))
    
     static let femtomolar = UnitConcentrationMass(symbol: "fM", converter: UnitConverterLinear(coefficient: 10 ^^ -15))
    
     static let attomolar = UnitConcentrationMass(symbol: "aM", converter: UnitConverterLinear(coefficient: 10 ^^ -18))
    
     static let zeptomolar = UnitConcentrationMass(symbol: "zM", converter: UnitConverterLinear(coefficient: 10 ^^ -21))
    
     static let yoctomolar = UnitConcentrationMass(symbol: "yM", converter: UnitConverterLinear(coefficient: 10 ^^ -24))
    
    
    
     static let allDilutionUnits = [
        
        (unit: UnitConcentrationMass.molar, description: "M"),
        
        (unit:UnitConcentrationMass.millimolar, description: "mM"),
        
        (unit:UnitConcentrationMass.micromolar,description: "µM"),
        
        (unit:UnitConcentrationMass.nanomolar,description: "nM"),
        
        (unit:UnitConcentrationMass.picomolar,description: "pM"),
        
        (unit:UnitConcentrationMass.femtomolar,description: "fM"),
        
        (unit:UnitConcentrationMass.attomolar,description: "aM"),
        
        (unit:UnitConcentrationMass.zeptomolar,description: "zM"),
        
        (unit:UnitConcentrationMass.yoctomolar,description: "yM")]
    
    
    
    
    
    func baseUnit() -> UnitConcentrationMass {
        
        return UnitConcentrationMass.molar
        
    }
    
}
// trying to add percentage
public extension UnitDispersion {
    
    
    
    static let percent = UnitDispersion(symbol: "%", converter: UnitConverterLinear(coefficient: 1))
    
    static let partsPerMillion = UnitDispersion(symbol: "ppm", converter: UnitConverterLinear(coefficient: 10 ^^ 4))
    
    static let partsPerBillion = UnitDispersion(symbol: "ppb", converter: UnitConverterLinear(coefficient: 10 ^^ 7))
    
    
    static let allDilutionUnits = [
        
        (unit: UnitDispersion.percent, description: "%"),
        
        (unit:UnitDispersion.partsPerMillion, description: "ppm"),
        
        (unit:UnitDispersion.partsPerBillion,description: "ppb")]
    
    
    
    
    
    func baseUnit() -> UnitDispersion {
        
        return UnitDispersion.percent
        
    }
    
}

