//
//  Correlation.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 12/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation

public enum Fitting: String, Codable{
    case linear
}



class Correlation : Codable , Equatable{

    
    var rCoefficient  = 0.0
    var labels = [String]()
    var components = [String:Double]()
    var formula : String = ""
    var type : Fitting = .linear
    var description : String {
        get {
            var description = ""
            switch type {
            case .linear:
                let slopeNumber = (self.components[ComponentLabels.slope])!
                let interceptNumber = (self.components[ComponentLabels.intercept])!
                
                let coefFormatter = NumberFormatter()
                coefFormatter.minimumIntegerDigits = 1
                coefFormatter.maximumFractionDigits = 3
                
                
                
                description = "y = \((coefFormatter.string(from: NSNumber(value:slopeNumber)))!) * x + \((coefFormatter.string(from: NSNumber(value:interceptNumber)))!)"
            }
            return description
        }
    }
    
    
    init(type: Fitting){
        switch type {
        case .linear:
            self.formula = "Y - \(ComponentLabels.intercept) / \(ComponentLabels.slope) ="
            self.labels.append(ComponentLabels.slope)
            self.labels.append(ComponentLabels.intercept)
//            self.components["slope"] = 0.0
//            self.components["intercept"] = 0.0
        }
        for label in self.labels {
            self.components.updateValue(0.0, forKey: label)
        }

    }
    
    public func convertToFormula() -> ([Step], [String:Double]){
        // return the relevant info so it can be converted into a useful experiment data
        
        let formula = Experiment.stepsFromFormula(formula: self.formula)
        var constants = [String:Double]()
        for component in components {
            constants.updateValue(component.value, forKey: component.key)
        }
        
        return (formula, constants)
    }
    
    //MARK: - Equatable Protocol
    static func ==(lhs: Correlation, rhs: Correlation) -> Bool {
        let result = true
        guard lhs.components == rhs.components else {
            return false
        }
        guard lhs.type == rhs.type else  {
            return false
        }
        
        return result
    }

}
