//
//  Input.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 3/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation

public class Input : Codable {
    
    // let's pack all the initial preparations done to the sample
    
    var label = ""
    var measure : Measurement<Unit>
    var shared = false
    
    
    init(){
        self.label = ""
        self.measure = Measurement(value:0.0, unit: UnitMass.kilograms)
    }
    
    // We are either measuring volumes or mass here
    init(label: String, measure: Measurement<UnitMass>){
        self.label = label
        self.measure = Measurement(value:measure.value, unit: measure.unit)
    }
    init(label: String, measure: Measurement<UnitVolume>){
        self.label = label
        self.measure = Measurement(value:measure.value, unit: measure.unit)
    }
    init(label: String, measure: Measurement<UnitConcentrationMass>){
        self.label = label
        self.measure = Measurement(value:measure.value, unit: measure.unit)
    }
    init(label: String, measure: Measurement<UnitDispersion>){
        self.label = label
        self.measure = Measurement(value:measure.value, unit: measure.unit)
    }


    
    init(fromVariable variable: Variable, withValue value: Double) {
        self.label = variable.label
        switch variable.dimension {
        case .mass:
            self.measure = Measurement(value: value , unit: UnitMass.grams)
        case .volume:
            self.measure = Measurement(value: value, unit: UnitVolume.milliliters)
        case .concentration:
            self.measure = Measurement(value: value, unit: UnitConcentrationMass.molar)
        case .regression:
            self.measure = Measurement(value: value, unit: UnitPower.gigawatts)
        case .percentage :
            self.measure = Measurement(value: value, unit: UnitDispersion.percent)
}
        self.shared = variable.shared
    }
    
}
