//
//  Regression.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 12/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation

class Regression : Codable , Equatable{
    
    var points = [Datapoint]()
    var dataUnits = [String:String]()
    var dataLabels = [String:String]()
    var correlation = Correlation(type: .linear)
    
    public func generateModel(data: [Datapoint]){
        self.points = data
        guard points.count > 0 else {
            print("No data points to fit")
            return
        }
        let nValue = Double(points.count)
        let meanX = points.reduce(0) {$0 + $1.x} / nValue
        let meanY = points.reduce(0) {$0 + $1.y} / nValue
        
        let aCoef = points.reduce(0) { $0 + ($1.x - meanX) * ($1.y - meanY) }
        
        let bCoef = points.reduce(0) { $0 + pow($1.x - meanX, 2) }
        // y = mx + c
        
        let m = aCoef / bCoef
        let c = meanY - m * meanX
        
        self.correlation.components.updateValue(m, forKey: ComponentLabels.slope)
        self.correlation.components.updateValue(c, forKey: ComponentLabels.intercept)
        self.correlation.rCoefficient = self.calculateLinearRegressionRcoef(slope: m, intercept: c)

    }
    public func generateModel(){
        guard points.count > 0 else {
            print("No data points to fit")
            return
        }
        // should switch on correlation.type linear fixed for the moment
        let nValue = Double(points.count)
        let meanX = points.reduce(0) {$0 + $1.x} / nValue
        let meanY = points.reduce(0) {$0 + $1.y} / nValue
        
        let aCoef = points.reduce(0) { $0 + ($1.x - meanX) * ($1.y - meanY) }
        
        let bCoef = points.reduce(0) { $0 + pow($1.x - meanX, 2) }
        // y = mx + c
        
        let m = aCoef / bCoef
        let c = meanY - m * meanX
        
        self.correlation.components.updateValue(m, forKey: ComponentLabels.slope)
        self.correlation.components.updateValue(c, forKey: ComponentLabels.intercept)
        self.correlation.rCoefficient = self.calculateLinearRegressionRcoef(slope: m, intercept: c)
        
    }
    
    func calculateLinearRegressionRcoef(slope: Double, intercept: Double) -> Double {
        
        //according to https://onlinecourses.science.psu.edu/stat501/node/255
        var ssr = 0.0
        var sse = 0.0
        var ssto = 0.0
        let nValue = Double(points.count)
        let meanY = points.reduce(0) {$0 + $1.y} / nValue

        for point in points {
            // predict the y value
            let yPredicted = slope * point.x + intercept
            // SSE (y_i - y^_i)^2
            let sseDiff =  point.y - yPredicted
            // SSR (y^_i - y_mean)^2
            let ssrDiff =  yPredicted - meanY
            // SSTO (y_i - y_mean)^2
            let sstoDiff =  point.y - meanY
            

            // square values
            let sseDiffSquared = pow(sseDiff, 2)
            let ssrDiffSquared = pow(ssrDiff, 2)
            let sstoDiffSquared = pow(sstoDiff, 2)
            
            // acumulate
            sse = sse + sseDiffSquared
            ssr = ssr + ssrDiffSquared
            ssto = ssto + sstoDiffSquared
        }
        let result = ssr / ssto
        return result
    }
    //MARK: - Equatable Protocol
    static func ==(lhs: Regression, rhs: Regression) -> Bool {
        let result = true
        // assume equal and return false if any of this verifications fails
        guard lhs.points == rhs.points else {
            return false
        }
        guard lhs.dataLabels == rhs.dataLabels else {
            return false
        }
        guard lhs.dataUnits == rhs.dataUnits else {
            return false
        }
        guard lhs.correlation == rhs.correlation else {
            return false
        }
        
        return result
    }

}
