//
//  Session.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 12/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation


public class Session : Codable , Equatable{
    
    var startDate = Date()
    var experimentType : Experiment = Experiment()
    var series = [Serie]()
    var description = "Description"
    var currentSerie = -1
    
    // We transported the substances to appDelegate
    //    var substances = [Solution]()

    public func copySessionToNew() -> Session {
        let copyCat = Session()
        copyCat.startDate = Date()
        copyCat.experimentType = self.experimentType
        copyCat.checkSeries()
        // copy the shared inputs
        let sharedInputs = self.series.first!.getInputs().filter({$0.shared})
        for input in sharedInputs {
            copyCat.series.first!.addInput(input)
        }
        copyCat.description = self.description
//        copyCat.series = [Serie]()
//        copycat.substances = self.substances
        return copyCat
    }
    
    public class func newSessionWithExperimentType(_ expType: Experiment, name: String? = nil) -> Session {
        let newSession = Session()
        newSession.description = name ?? expType.name
        newSession.experimentType = expType
        newSession.startDate = Date()
        newSession.checkSeries()
        
        return newSession
    }
    
    public func addSeries(){
        self.series.append(Serie())
        self.currentSerie = self.series.count - 1 
        
    }
    
    private func  checkSeries() {
        if self.series.count == 0{
            self.addSeries()
        }
    }
    
    public func switchToSeries(newSeries: Int) -> Bool{
        checkSeries()

        switch newSeries {
        case 0 ..< self.series.count:
            self.currentSerie = newSeries
            return true
        default:
            return false
        }
        
    }
    
    public func addInput(_ input: Input, newSeries: Bool = false ) {
        checkSeries()
        switch newSeries {
        case true:
            self.addSeries()
            
        default:
            break
        }
        self.series[currentSerie].addInput(input)
    }
    
    public func cleanShared(_ serie: Int = 0){
        checkSeries()
        switch serie {
        case 0:
            self.series[serie].cleanShared()
        default:
            break
        }
    }
    
    // MARK: String export methods for QR encoding
    
    public func exportString() -> String {
        
        let encoder = JSONEncoder()
        encoder.dateEncodingStrategy = .iso8601
        var data = Data()
        do {
            data = try encoder.encode(self)
        } catch  {
            print("Failure encoding \(String(describing:self))")
        }
        let output = String(data:data, encoding: .utf8) ?? ""
        
        return output
        
    }
    
    public func decodeFrom(string: String){
        let decoder = JSONDecoder()
        decoder.dateDecodingStrategy = .iso8601
        let data = string.data(using: .utf8)
        do {
            let newSelf = try decoder.decode(Session.self, from: data!)
            self.startDate = newSelf.startDate
            self.experimentType = newSelf.experimentType
            self.series = newSelf.series
            self.description = newSelf.description
            self.currentSerie = newSelf.currentSerie
            
        } catch  {
            print("Error thrown when decoding")
        }
    }

    
    // MARK: - Equatable code
    public static func ==(lhs: Session, rhs: Session) -> Bool {
        return ((lhs.startDate == rhs.startDate) && (lhs.description == rhs.description) && (lhs.experimentType == rhs.experimentType))
        
    }


    
}
