//
//  Substance.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 3/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import Foundation


class Solution : Codable {
    
    var kind : Kind = .other
    var substance = "" {
        didSet {
            if substance != "" {
                self.molecularWeight = Solution.getMW(forCompound: substance)
            }
        }
    }
    var realConcentration : Measurement<UnitConcentrationMass>?
    var molecularWeight = 1.0
    var factor = 1.0 {
        didSet {
            if labelConcentration > 0.0 {
                let conc = labelConcentration * factor
                self.realConcentration = Measurement(value: conc, unit: .molar)
                
            }
        }
    }
    var labelConcentration = 0.0 {
        didSet {
            if labelConcentration > 0.0 {
                let conc = labelConcentration * factor
                self.realConcentration = Measurement(value: conc, unit: .molar)
                
            }
        }
    }
    var equivalents = 1

    init(){
        self.substance = ""
        self.realConcentration = nil
        self.molecularWeight = 1.0
        self.factor = 1.0
        self.equivalents = 1
    }
    
    public init(substance: String , atMolarity: Double, factor: Double? = 1.0, equivalence: Int? = 1, MW: Double?, kind: Kind? = .other) {
        
        // Substance is not empty
        self.substance = substance
        self.factor = factor ?? 1.000
        self.equivalents = equivalence ?? 1
        self.labelConcentration = atMolarity
        self.molecularWeight = MW ?? Solution.getMW(forCompound: self.substance)
        let realConcentration = labelConcentration * self.factor
        // MW can be obtained from
        // https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/name/H2O/JSON
        
        self.realConcentration = Measurement(value: realConcentration, unit: .molar)
        self.kind = kind ?? .other
        
//        print(self.MW)
        
    }
    
    class func getMW(forCompound: String) -> Double {
        let semaphore = DispatchSemaphore(value:0)
        var MW = 0.0
        let getMWEndpoint = "https://pubchem.ncbi.nlm.nih.gov/rest/pug/compound/name/\(forCompound)/JSON"
        guard let url = URL(string: getMWEndpoint) else {
            print("Error generating URL")
            return MW
        }
        let urlRequest = URLRequest(url:url)
        
        let session = URLSession.shared
        
        let task = session.dataTask(with: urlRequest){
            data, response, error in
            
            guard let responseData = data else {
                print("Error: Did not receive data")
                return
                
            }
            do {
                let jsonData = try JSON(data: responseData)
                let propsPath : [JSONSubscriptType] = ["PC_Compounds", 0, "props"]
                let props = jsonData[propsPath]
                // we have the props here, we have to locate the one that has urn.label == "Molecular Weight"
                
                for (_,property):(String, JSON) in props {
                    //Do something you want
                    let labelPath : [JSONSubscriptType] = ["urn", "label"]
                    let valuePath : [JSONSubscriptType] = ["value", "fval"]
                    let label = property[labelPath]
                    if label == "Molecular Weight" {
                        let value = property[valuePath].doubleValue
                        MW = value
                    }
                }
                
            } catch {
                print("Error: Trying to convert data to JSON(GetMW)")
                return
            }
            semaphore.signal()
        }
        
        task.resume()
        semaphore.wait()
        
        
        return MW
    }

    func copySolutionToNew() -> Solution {
        let newSolution = Solution(substance: self.substance, atMolarity: self.labelConcentration, factor: self.factor, equivalence: self.equivalents, MW: self.molecularWeight, kind: self.kind)
        return newSolution
    }
    
}




