//
//  SharedInputsPresentation.swift
//  MeasurementHelper
//
//  Created by Carlos Barrera on 17/11/17.
//  Copyright © 2017 Carlos Barrera. All rights reserved.
//

import UIKit

@IBDesignable class SharedInputsPresentation: UIStackView {
    
    @IBInspectable var labelSize : CGSize = CGSize(width: 20.0, height: 20.0)
    @IBInspectable var numberOfLabels : Int = 2 {
        didSet{
            self.removeInputs()
            self.setupInputs()
        }
    }

    private var lines = [UIStackView]()
    

    
    // MARK: Initialization
    override init(frame: CGRect) {
        super.init(frame: frame)
        setupInputs()
    }
    
    required init(coder: NSCoder) {
        super.init(coder: coder)
        setupInputs()
    }

    /*
    // Only override draw() if you perform custom drawing.
    // An empty implementation adversely affects performance during animation.
    override func draw(_ rect: CGRect) {
        // Drawing code
    }
    */
    private func getHorizontalLineWithInput(input: Input)-> UIStackView {
        let stack = UIStackView()
        stack.alignment = UIStackViewAlignment.firstBaseline
        let label = UILabel()
        label.text = input.label
        label.heightAnchor.constraint(equalToConstant: labelSize.height).isActive = true
        let field = UITextField()
        field.text = input.measure.value.description
        field.borderStyle = UITextBorderStyle.bezel
        field.heightAnchor.constraint(equalToConstant: labelSize.height).isActive = true
        let units = UILabel()
        units.text = input.measure.unit.symbol
        units.heightAnchor.constraint(equalToConstant: labelSize.height).isActive = true
        
        stack.addArrangedSubview(label)
        stack.addArrangedSubview(field)
        stack.addArrangedSubview(units)
        return stack
    }
    
    // MARK: Private Methods
    private func setupInputs(inputs: [Input]? = [Input]() ) {
        
        
//        let input = Input(label: "Measure 1", measure: Measurement<UnitMass>(value:2, unit:.kilograms))
//
        var myInputs = inputs!
        if myInputs.count == 0 {
            for i in 0..<numberOfLabels {
                myInputs.append(Input(label: "Measure \(i+1)", measure: Measurement<UnitMass>(value:Double(i+1), unit:.kilograms)))

            }
        }
        for input in myInputs {
            // create the Line
            let line = getHorizontalLineWithInput(input: input)
            lines.append(line)
            
            // add the constraints
//            line.translatesAutoresizingMaskIntoConstraints = false
            line.heightAnchor.constraint(equalToConstant: labelSize.height).isActive = true
            line.widthAnchor.constraint(equalToConstant: labelSize.width).isActive = true

            //add the label to the stack
            addArrangedSubview(line)

        }
//        alignment = .fill
//        distribution = .equalCentering

        
    }
    public func setInputs(inputs: [Input]){
        self.removeInputs()
        self.setupInputs(inputs:inputs)
        
    }
    
    private func removeInputs() {
        for line in lines {
            removeArrangedSubview(line)
            line.removeFromSuperview()
        }

    }

}
